@@ -1,14 +1,15 @@
 import logging
 
 from django.conf import settings
-from django.views.decorators.cache import never_cache
-from django.contrib.auth.decorators import login_required
-from django.utils.decorators import method_decorator
-from django.core.exceptions import ObjectDoesNotExist
-from django.contrib.auth import logout
 from django.contrib import messages
-from django.utils.translation import ugettext_lazy as _
+from django.contrib.auth import logout
+from django.contrib.auth.decorators import login_required
+from django.core.exceptions import ObjectDoesNotExist
+from django.core.urlresolvers import get_callable
 from django.core.urlresolvers import reverse_lazy
+from django.utils.decorators import method_decorator
+from django.utils.translation import ugettext_lazy as _
+from django.views.decorators.cache import never_cache
 
 from mama_cas.compat import SiteProfileNotAvailable
 from mama_cas.models import ServiceTicket
@@ -162,31 +163,36 @@ class CustomAttributesMixin(object):
     """
     def get_custom_attributes(self, ticket):
         """
-        Build a list of user attributes from the ``User`` and/or user
-        profile object. The attributes are defined with two settings:
+        Build a list of user attributes from the ``User`` object, the
+        user profile object or a custom callback callable. Attributes
+        are specified with one or more settings:
 
         ``MAMA_CAS_USER_ATTRIBUTES``
-            A list of name and ``User`` attribute pairs. The name can
+            A dict of name and ``User`` attribute values. The name can
             be any meaningful string, while the attribute must
             correspond with an attribute on the ``User`` object.
 
         ``MAMA_CAS_PROFILE_ATTRIBUTES``
-            A list of name and user profile attribute pairs. The name
+            A dict of name and user profile attribute values. The name
             can be any meaningful string, while the attribute must
             correspond with an attribute on the user profile object.
 
-        One or both of the settings variables may be used, with all
-        items returned as a single list. Ordering is not guaranteed.
+        ``MAMA_CAS_ATTRIBUTES_CALLBACK``
+            A string representation of a custom callable that returns
+            a dict containing attributes. The callable is provided a
+            single argument of the ``User``.
+
+        All attributes are returned as a single dictionary.
         """
         if not ticket:
             return None
         user = ticket.user
-        attributes = []
+        attributes = {}
 
         user_attr_list = getattr(settings, 'MAMA_CAS_USER_ATTRIBUTES', {})
         for name, attr in user_attr_list.items():
             try:
-                attributes.append((name, getattr(user, attr)))
+                attributes[name] = getattr(user, attr)
             except AttributeError:
                 logger.error("User has no attribute named '%s'" % attr)
 
@@ -199,10 +205,15 @@ class CustomAttributesMixin(object):
                                         'MAMA_CAS_PROFILE_ATTRIBUTES', {})
             for name, attr in profile_attr_list.items():
                 try:
-                    attributes.append((name, getattr(profile, attr)))
+                    attributes[name] = getattr(profile, attr)
                 except AttributeError:
                     logger.error("Profile has no attribute named '%s'" % attr)
 
+        callback_str = getattr(settings, 'MAMA_CAS_ATTRIBUTES_CALLBACK', None)
+        if callback_str is not None:
+            callback = get_callable(callback_str)
+            attributes.update(callback(user))
+
         return attributes
 
 
@@ -213,18 +224,23 @@ class LogoutUserMixin(object):
     def logout_user(self, request):
         """
         End a single sign-on session for the current user. This process
-        occurs in two steps:
+        occurs in three steps:
 
         1. Consume all valid tickets created for the user.
 
-        2. Call logout() to end the session and purge all session data.
+        2. (Optional) Send single sign-out requests to services.
+
+        3. Call logout() to end the session and purge all session data.
         """
         if request.user.is_authenticated():
             ServiceTicket.objects.consume_tickets(request.user)
             ProxyTicket.objects.consume_tickets(request.user)
             ProxyGrantingTicket.objects.consume_tickets(request.user)
 
+            if getattr(settings, 'MAMA_CAS_ENABLE_SINGLE_SIGN_OUT', False):
+                ServiceTicket.objects.request_sign_out(request.user)
+
             logger.info("Single sign-on session ended for %s" % request.user)
             logout(request)
-            messages.success(request,
-                             _("You have been successfully logged out"))
+            msg = _("You have been successfully logged out")
+            messages.success(request, msg)
