@@ -1,41 +1,38 @@
 from __future__ import unicode_literals
 
+from mock import patch
+
 try:
     from urllib.parse import quote
 except ImportError:  # pragma: no cover
     from urllib import quote
 
-from django.conf import settings
 from django.core.urlresolvers import reverse
 from django.test import TestCase
-from django.utils import unittest
+from django.test.utils import override_settings
 
-from mama_cas.compat import get_user_model
+from .factories import UserFactory
+from .factories import ProxyGrantingTicketFactory
+from .factories import ProxyTicketFactory
+from .factories import ServiceTicketFactory
+from .factories import ConsumedServiceTicketFactory
 from mama_cas.forms import LoginForm
-from mama_cas.forms import LoginFormWarn
-from mama_cas.models import ProxyGrantingTicket
-from mama_cas.models import ProxyTicket
 from mama_cas.models import ServiceTicket
 
 
-XMLNS = '{http://www.yale.edu/tp/cas}'
-
-
 class LoginViewTests(TestCase):
     """
     Test the ``LoginView`` view.
     """
     user_info = {'username': 'ellen',
+                 'password': 'mamas&papas'}
+    warn_info = {'username': 'ellen',
                  'password': 'mamas&papas',
-                 'email': 'ellen@example.com'}
+                 'warn': 'on'}
     service_url = 'http://www.example.com/'
 
     def setUp(self):
-        """
-        Initialize the environment for each test.
-        """
-        user = get_user_model()
-        self.user = user.objects.create_user(**self.user_info)
+        self.user = UserFactory()
 
     def test_login_view(self):
         """
@@ -89,9 +86,8 @@ class LoginViewTests(TestCase):
         user, create a ``ServiceTicket`` and redirect to the supplied
         service URL with the ticket included.
         """
-        params = self.user_info.copy()
-        params.update({'service': self.service_url})
-        response = self.client.post(reverse('cas_login'), params)
+        url = reverse('cas_login') + "?service=%s" % self.service_url
+        response = self.client.post(url, self.user_info)
         self.assertEqual(self.client.session['_auth_user_id'], self.user.pk)
         self.assertEqual(ServiceTicket.objects.count(), 1)
         st = ServiceTicket.objects.latest('id')
@@ -137,50 +133,57 @@ class LoginViewTests(TestCase):
         self.assertTrue(response['Location'].startswith(self.service_url))
         self.assertTrue(st.ticket in response['Location'])
 
+    @override_settings(MAMA_CAS_ALLOW_AUTH_WARN=True)
+    def test_login_view_warn_session(self):
+        """
+        When a user logs in with the warn parameter present, the user's
+        session should contain a ``warn`` attribute.
+        """
+        self.client.post(reverse('cas_login'), self.warn_info)
+        self.assertEqual(self.client.session.get('warn'), True)
 
+    @override_settings(MAMA_CAS_ALLOW_AUTH_WARN=True)
+    def test_login_view_warn_auth_redirect(self):
+        """
+        When a logged in user requests a ``ServiceTicket`` and the
+        ``warn`` attribute is set, it should redirect to the warn view
+        with the appropriate parameters.
+        """
+        self.client.post(reverse('cas_login'), self.warn_info)
+        quote_url = quote(self.service_url, '')
+        query_str = "?service=%s" % quote_url
+        response = self.client.get(reverse('cas_login') + query_str)
+        self.assertTrue(reverse('cas_warn') in response['Location'])
+        self.assertTrue("service=%s" % quote_url in response['Location'])
+        self.assertTrue('ticket=ST-' in response['Location'])
+
+    @override_settings(MAMA_CAS_ALLOW_AUTH_WARN=True)
+    def test_login_view_warn_auth_gateway_redirect(self):
+        """
+        When a logged in user requests a ``ServiceTicket`` with the
+        gateway parameter and the ``warn`` attribute is set, it should
+        redirect to the warn view with the appropriate parameters.
+        """
+        self.client.post(reverse('cas_login'), self.warn_info)
+        quote_url = quote(self.service_url, '')
+        query_str = "?gateway=true&service=%s" % quote_url
+        response = self.client.get(reverse('cas_login') + query_str)
+        self.assertTrue(reverse('cas_warn') in response['Location'])
+        self.assertTrue("service=%s" % quote_url in response['Location'])
+        self.assertTrue('ticket=ST-' in response['Location'])
+
+
+@override_settings(MAMA_CAS_ALLOW_AUTH_WARN=True)
 class WarnViewTests(TestCase):
     """
     Test the ``WarnView`` view.
     """
     user_info = {'username': 'ellen',
-                 'password': 'mamas&papas',
-                 'email': 'ellen@example.com'}
-    service_url = 'http://www.example.com/'
+                 'password': 'mamas&papas'}
+    url = 'http://www.example.com'
 
     def setUp(self):
-        """
-        Initialize the environment for each test.
-        """
-        user = get_user_model()
-        self.user = user.objects.create_user(**self.user_info)
-
-    def test_warn_view(self):
-        """
-        When called with no parameters and no logged in user, a ``GET``
-        request to the view should simply redirect to the login view.
-        """
-        response = self.client.get(reverse('cas_warn'))
-        self.assertRedirects(response, reverse('cas_login'))
-        self.assertTrue('Cache-Control' in response)
-        self.assertEqual(response['Cache-Control'], 'max-age=0')
-
-    def test_warn_view_redirect(self):
-        """
-        When a user logs in with the warn parameter present, the user's
-        session should contain a ``warn`` attribute and a
-        ``ServiceTicket`` request to the credential requestor should
-        redirect to the warn view.
-        """
-        response = self.client.get(reverse('cas_login'))
-        # Only continue the test if the required form class is in use
-        if isinstance(response.context['form'], LoginFormWarn):
-            form_data = self.user_info.copy()
-            form_data.update({'warn': 'true'})
-            response = self.client.post(reverse('cas_login'), form_data)
-            query_str = "?service=%s" % quote(self.service_url, '')
-            response = self.client.get(reverse('cas_login') + query_str)
-            self.assertEqual(self.client.session.get('warn'), True)
-            self.assertRedirects(response, reverse('cas_warn') + query_str)
+        self.user = UserFactory()
 
     def test_warn_view_display(self):
         """
@@ -190,23 +193,24 @@ class WarnViewTests(TestCase):
         """
         self.client.login(username=self.user_info['username'],
                           password=self.user_info['password'])
-        query_str = "?service=%s" % quote(self.service_url, '')
+        st = ServiceTicketFactory()
+        query_str = "?service=%s&ticket=%s" % (quote(self.url, ''), st.ticket)
         response = self.client.get(reverse('cas_warn') + query_str)
-        self.assertContains(response, self.service_url)
+        self.assertContains(response, self.url, count=3)
+        self.assertContains(response, st.ticket)
         self.assertTemplateUsed(response, 'mama_cas/warn.html')
 
-    def test_warn_view_warned(self):
+    def test_warn_view_anonymous_user(self):
         """
-        When a logged in user submits the form on the warn view, the
-        user should be redirected to the login view with the ``warned``
-        parameter present.
+        When a user is not logged in, a ``GET`` request to the view
+        should redirect to the login view.
         """
-        self.client.login(username=self.user_info['username'],
-                          password=self.user_info['password'])
-        response = self.client.post(reverse('cas_warn'))
-        self.assertRedirects(response, reverse('cas_login') + '?warned=true')
+        response = self.client.get(reverse('cas_warn'))
+        self.assertRedirects(response, reverse('cas_login'))
 
 
+@override_settings(MAMA_CAS_VALID_SERVICES=('.*\.example\.com',))
+@override_settings(MAMA_CAS_FOLLOW_LOGOUT_URL=False)
 class LogoutViewTests(TestCase):
     """
     Test the ``LogoutView`` view.
@@ -216,24 +220,7 @@ class LogoutViewTests(TestCase):
                  'email': 'ellen@example.com'}
 
     def setUp(self):
-        """
-        Initialize the environment for each test.
-        """
-        user = get_user_model()
-        self.user = user.objects.create_user(**self.user_info)
-        self.old_valid_services = getattr(settings,
-                                          'MAMA_CAS_VALID_SERVICES', ())
-        settings.MAMA_CAS_VALID_SERVICES = ('.*\.example\.com',)
-        self.old_follow_url = getattr(settings,
-                                      'MAMA_CAS_FOLLOW_LOGOUT_URL', False)
-        settings.MAMA_CAS_FOLLOW_LOGOUT_URL = False
-
-    def tearDown(self):
-        """
-        Undo any modifications made to the test environment.
-        """
-        settings.MAMA_CAS_VALID_SERVICES = self.old_valid_services
-        settings.MAMA_CAS_FOLLOW_LOGOUT_URL = self.old_follow_url
+        self.user = UserFactory()
 
     def test_logout_view(self):
         """
@@ -264,14 +251,13 @@ class LogoutViewTests(TestCase):
         self.assertRedirects(response, reverse('cas_login'))
         self.assertFalse('_auth_user_id' in self.client.session)
 
+    @override_settings(MAMA_CAS_FOLLOW_LOGOUT_URL=True)
     def test_logout_view_follow_url(self):
         """
         When called with a logged in user and MAMA_CAS_FOLLOW_LOGOUT_URL
         is set to ``True``, a ``GET`` request to the view should log the
         user out and redirect to the supplied URL.
         """
-        settings.MAMA_CAS_FOLLOW_LOGOUT_URL = True
-
         response = self.client.post(reverse('cas_login'), self.user_info)
         query_str = '?url=http://www.example.com'
         response = self.client.get(reverse('cas_logout') + query_str)
@@ -279,7 +265,23 @@ class LogoutViewTests(TestCase):
         self.assertEqual(response['Location'], 'http://www.example.com')
         self.assertFalse('_auth_user_id' in self.client.session)
 
+    @override_settings(MAMA_CAS_ENABLE_SINGLE_SIGN_OUT=True)
+    def test_logout_single_sign_out(self):
+        """
+        When called with a logged in user and MAMA_CAS_ENABLE_SINGLE_SIGN_OUT
+        is set to ``True``, a ``GET`` request to the view should issue
+        a POST request for each service accessed by the user.
+        """
+        ConsumedServiceTicketFactory()
+        ConsumedServiceTicketFactory()
+        self.client.post(reverse('cas_login'), self.user_info)
+        query_str = '?url=http://www.example.com'
+        with patch('requests.post') as mock:
+            self.client.get(reverse('cas_logout') + query_str)
+            self.assertEqual(mock.call_count, 2)
 
+
+@override_settings(MAMA_CAS_VALID_SERVICES=('.*\.example\.com',))
 class ValidateViewTests(TestCase):
     """
     Test the ``ValidateView`` view.
@@ -287,25 +289,7 @@ class ValidateViewTests(TestCase):
     service_url = 'http://www.example.com/'
 
     def setUp(self):
-        """
-        Initialize the environment for each test.
-        """
-        user = get_user_model()
-        self.user = user.objects.create_user('ellen',
-                                             password='mamas&papas',
-                                             email='ellen@example.com')
-        self.st = ServiceTicket.objects.create_ticket(service=self.service_url,
-                                                      user=self.user)
-
-        self.old_valid_services = getattr(settings,
-                                          'MAMA_CAS_VALID_SERVICES', ())
-        settings.MAMA_CAS_VALID_SERVICES = ('http://.*\.example\.com/',)
-
-    def tearDown(self):
-        """
-        Undo any modifications made to the test environment.
-        """
-        settings.MAMA_CAS_VALID_SERVICES = self.old_valid_services
+        self.st = ServiceTicketFactory()
 
     def test_validate_view(self):
         """
@@ -367,6 +351,11 @@ class ValidateViewTests(TestCase):
         self.assertEqual(response.get('Content-Type'), 'text/plain')
 
 
+@override_settings(MAMA_CAS_VALID_SERVICES=('.*\.example\.com',))
+@override_settings(MAMA_CAS_USER_ATTRIBUTES={'givenName': 'first_name',
+                                             'sn': 'last_name',
+                                             'email': 'email',
+                                             'test': 'invalid'})
 class ServiceValidateViewTests(TestCase):
     """
     Test the ``ServiceValidateView`` view.
@@ -375,35 +364,7 @@ class ServiceValidateViewTests(TestCase):
     pgt_url = 'https://www.example.com/'
 
     def setUp(self):
-        """
-        Initialize the environment for each test.
-        """
-        user = get_user_model()
-        self.user = user.objects.create_user('ellen',
-                                             email='ellen@example.com',
-                                             password='mamas&papas')
-        self.user.first_name = 'Ellen'
-        self.user.last_name = 'Cohen'
-        self.user.save()
-        self.st = ServiceTicket.objects.create_ticket(service=self.service_url,
-                                                      user=self.user)
-
-        self.old_user_attributes = getattr(settings,
-                                           'MAMA_CAS_USER_ATTRIBUTES', {})
-        settings.MAMA_CAS_USER_ATTRIBUTES = {'givenName': 'first_name',
-                                             'sn': 'last_name',
-                                             'email': 'email',
-                                             'test': 'invalid'}
-        self.old_valid_services = getattr(settings,
-                                          'MAMA_CAS_VALID_SERVICES', ())
-        settings.MAMA_CAS_VALID_SERVICES = ('http://.*\.example\.com/',)
-
-    def tearDown(self):
-        """
-        Undo any modifications made to the test environment.
-        """
-        settings.MAMA_CAS_USER_ATTRIBUTES = self.old_user_attributes
-        settings.MAMA_CAS_VALID_SERVICES = self.old_valid_services
+        self.st = ServiceTicketFactory()
 
     def test_service_validate_view(self):
         """
@@ -472,21 +433,19 @@ class ServiceValidateViewTests(TestCase):
         response = self.client.get(reverse('cas_service_validate') + query_str)
         self.assertContains(response, 'INVALID_TICKET')
 
-    @unittest.skipIf(pgt_url == 'https://www.example.com/',
-                     'set pgt_url to a valid HTTPS URL')
     def test_service_validate_view_pgturl(self):
         """
         When called with correct parameters and a ``pgtUrl`` parameter,
         a ``GET`` request to the view should return a validation
         success and also attempt to create a ``ProxyGrantingTicket``.
-
-        NOTE: this test will fail unless ``pgt_url`` is configured with
-        a valid proxy callback URL.
         """
         query_str = "?service=%s&ticket=%s&pgtUrl=%s" % (self.service_url,
                                                          self.st.ticket,
                                                          self.pgt_url)
-        response = self.client.get(reverse('cas_service_validate') + query_str)
+        url = reverse('cas_service_validate') + query_str
+        with patch('requests.get') as mock:
+            mock.return_value.status_code = 200
+            response = self.client.get(url)
         self.assertContains(response, 'ellen')
         self.assertContains(response, 'proxyGrantingTicket')
 
@@ -503,19 +462,17 @@ class ServiceValidateViewTests(TestCase):
         self.assertContains(response, 'ellen')
         self.assertNotContains(response, 'proxyGrantingTicket')
 
+    @override_settings(MAMA_CAS_ATTRIBUTE_FORMAT='jasig')
     def test_service_validate_view_user_attributes(self):
         """
         When ``MAMA_CAS_USER_ATTRIBUTES`` is defined in the settings
         file, a service validation success should include the list of
         configured user attributes.
         """
-        attr_format = getattr(settings, 'MAMA_CAS_ATTRIBUTE_FORMAT', 'jasig')
-        settings.MAMA_CAS_ATTRIBUTE_FORMAT = 'jasig'
         query_str = "?service=%s&ticket=%s" % (self.service_url,
                                                self.st.ticket)
         response = self.client.get(reverse('cas_service_validate') + query_str)
         self.assertContains(response, 'attributes')
-        settings.MAMA_CAS_ATTRIBUTE_FORMAT = attr_format
 
     def test_service_validate_view_invalid_service_url(self):
         """
@@ -529,7 +486,23 @@ class ServiceValidateViewTests(TestCase):
         response = self.client.get(reverse('cas_service_validate') + query_str)
         self.assertContains(response, 'INVALID_SERVICE')
 
+    @override_settings(MAMA_CAS_ATTRIBUTES_CALLBACK='mama_cas.tests.callback.test_callback')
+    def test_service_validate_view_attributes_callback(self):
+        """
+        When a custom callback is defined in the settings file, a service
+        validation success should include the attributes that callback
+        returns.
+        """
+        query_str = "?service=%s&ticket=%s" % (self.service_url, self.st.ticket)
+        response = self.client.get(reverse('cas_service_validate') + query_str)
+        self.assertContains(response, '<cas:username>ellen</cas:username>')
 
+
+@override_settings(MAMA_CAS_VALID_SERVICES=('.*\.example\.com',))
+@override_settings(MAMA_CAS_USER_ATTRIBUTES={'givenName': 'first_name',
+                                             'sn': 'last_name',
+                                             'email': 'email',
+                                             'test': 'invalid'})
 class ProxyValidateViewTests(TestCase):
     """
     Test the ``ProxyValidateView`` view.
@@ -539,42 +512,9 @@ class ProxyValidateViewTests(TestCase):
     pgt_url = 'https://www.example.com/'
 
     def setUp(self):
-        """
-        Initialize the environment for each test.
-        """
-        user = get_user_model()
-        self.user = user.objects.create_user('ellen',
-                                             email='ellen@example.com',
-                                             password='mamas&papas')
-        self.userfirst_name = 'Ellen'
-        self.user.last_name = 'Cohen'
-        self.user.save()
-        self.st = ServiceTicket.objects.create_ticket(service=self.service_url,
-                                                      user=self.user)
-        self.pgt = ProxyGrantingTicket.objects.create_ticket(self.service_url,
-                                                             validate=False,
-                                                             user=self.user,
-                                                             granted_by_st=self.st)
-        self.pt = ProxyTicket.objects.create_ticket(service=self.service_url,
-                                                    user=self.user,
-                                                    granted_by_pgt=self.pgt)
-
-        self.old_user_attributes = getattr(settings,
-                                           'MAMA_CAS_USER_ATTRIBUTES', {})
-        settings.MAMA_CAS_USER_ATTRIBUTES = {'givenName': 'first_name',
-                                             'sn': 'last_name',
-                                             'email': 'email',
-                                             'test': 'invalid'}
-        self.old_valid_services = getattr(settings,
-                                          'MAMA_CAS_VALID_SERVICES', ())
-        settings.MAMA_CAS_VALID_SERVICES = ('http://.*\.example\.com/',)
-
-    def tearDown(self):
-        """
-        Undo any modifications made to the test environment.
-        """
-        settings.MAMA_CAS_USER_ATTRIBUTES = self.old_user_attributes
-        settings.MAMA_CAS_VALID_SERVICES = self.old_valid_services
+        self.st = ServiceTicketFactory()
+        self.pgt = ProxyGrantingTicketFactory()
+        self.pt = ProxyTicketFactory()
 
     def test_proxy_validate_view(self):
         """
@@ -638,8 +578,7 @@ class ProxyValidateViewTests(TestCase):
         ``ProxyTicket`` should be consumed and invalid for future
         validation attempts.
         """
-        query_str = "?service=%s&ticket=%s" % (self.service_url,
-                                               self.pt.ticket)
+        query_str = "?service=%s&ticket=%s" % (self.pt.service, self.pt.ticket)
         response = self.client.get(reverse('cas_proxy_validate') + query_str)
         self.assertContains(response, 'ellen')
         self.assertContains(response, 'http://www.example.com')
@@ -660,34 +599,29 @@ class ProxyValidateViewTests(TestCase):
         multiple proxies, they must be listed in reverse order of being
         accessed.
         """
-        pgt2 = ProxyGrantingTicket.objects.create_ticket(self.service_url,
-                                                         user=self.user,
-                                                         granted_by_pt=self.pt,
-                                                         validate=False)
-        pt2 = ProxyTicket.objects.create_ticket(service='http://ww2.example.com',
-                                                user=self.user,
-                                                granted_by_pgt=pgt2)
-        query_str = "?service=%s&ticket=%s" % ('http://ww2.example.com/', pt2)
+        pgt2 = ProxyGrantingTicketFactory(granted_by_pt=self.pt,
+                                          granted_by_st=None)
+        pt2 = ProxyTicketFactory(service='http://ww2.example.com',
+                                 granted_by_pgt=pgt2)
+        query_str = "?service=%s&ticket=%s" % (pt2.service, pt2.ticket)
         response = self.client.get(reverse('cas_proxy_validate') + query_str)
         self.assertContains(response, 'ellen')
         self.assertContains(response, 'http://ww2.example.com')
         self.assertContains(response, 'http://www.example.com')
 
-    @unittest.skipIf(pgt_url == 'https://www.example.com/',
-                     'set pgt_url to a valid HTTPS URL')
     def test_proxy_validate_view_pgturl(self):
         """
         When called with correct parameters and a ``pgtUrl`` parameter,
         a ``GET`` request to the view should return a validation
         success and also attempt to create a ``ProxyGrantingTicket``.
-
-        NOTE: this test will fail unless ``pgt_url`` is configured with
-        a valid proxy callback URL.
         """
         query_str = "?service=%s&ticket=%s&pgtUrl=%s" % (self.service_url,
                                                          self.pt.ticket,
                                                          self.pgt_url)
-        response = self.client.get(reverse('cas_proxy_validate') + query_str)
+        url = reverse('cas_proxy_validate') + query_str
+        with patch('requests.get') as mock:
+            mock.return_value.status_code = 200
+            response = self.client.get(url)
         self.assertContains(response, 'ellen')
         self.assertContains(response, 'proxyGrantingTicket')
 
@@ -704,19 +638,17 @@ class ProxyValidateViewTests(TestCase):
         self.assertContains(response, 'ellen')
         self.assertNotContains(response, 'proxyGrantingTicket')
 
+    @override_settings(MAMA_CAS_ATTRIBUTE_FORMAT='jasig')
     def test_proxy_validate_view_user_attributes(self):
         """
         When ``MAMA_CAS_USER_ATTRIBUTES`` is defined in the settings
         file, a proxy validation success should include the list of
         configured user attributes.
         """
-        attr_format = getattr(settings, 'MAMA_CAS_ATTRIBUTE_FORMAT', 'jasig')
-        settings.MAMA_CAS_ATTRIBUTE_FORMAT = 'jasig'
         query_str = "?service=%s&ticket=%s" % (self.service_url,
                                                self.st.ticket)
         response = self.client.get(reverse('cas_proxy_validate') + query_str)
         self.assertContains(response, 'attributes')
-        settings.MAMA_CAS_ATTRIBUTE_FORMAT = attr_format
 
     def test_proxy_validate_view_invalid_service_url(self):
         """
@@ -739,29 +671,8 @@ class ProxyViewTests(TestCase):
     invalid_service = 'http://www.example.org/'
 
     def setUp(self):
-        """
-        Create a valid user and service ticket for testing purposes.
-        """
-        user = get_user_model()
-        self.user = user.objects.create_user('ellen',
-                                             password='mamas&papas',
-                                             email='ellen@example.com')
-        self.st = ServiceTicket.objects.create_ticket(service=self.service_url,
-                                                      user=self.user)
-        self.pgt = ProxyGrantingTicket.objects.create_ticket(self.service_url,
-                                                             user=self.user,
-                                                             granted_by_st=self.st,
-                                                             validate=False)
-
-        self.old_valid_services = getattr(settings,
-                                          'MAMA_CAS_VALID_SERVICES', ())
-        settings.MAMA_CAS_VALID_SERVICES = ('http://.*\.example\.com/',)
-
-    def tearDown(self):
-        """
-        Undo any modifications made to settings.
-        """
-        settings.MAMA_CAS_VALID_SERVICES = self.old_valid_services
+        self.st = ServiceTicketFactory()
+        self.pgt = ProxyGrantingTicketFactory()
 
     def test_proxy_view(self):
         """
@@ -811,12 +722,12 @@ class ProxyViewTests(TestCase):
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.get('Content-Type'), 'text/xml')
 
+    @override_settings(MAMA_CAS_VALID_SERVICES=('.*\.example\.com',))
     def test_proxy_view_invalid_service_url(self):
         """
-        When ``MAMA_CAS_VALID_SERVICES`` is defined in the settings
-        file, a service string should be checked against the list of
-        valid services. If it does not match, a proxy authentication
-        failure should be returned.
+        When ``MAMA_CAS_VALID_SERVICES`` is defined, a service string
+        should be checked against the list of valid services. If it does
+        not match, a proxy authentication failure should be returned.
         """
         query_str = "?targetService=%s&pgt=%s" % (self.invalid_service,
                                                   self.pgt.ticket)
